@@ -8,7 +8,7 @@ import logging
 import re
 import time
 import weakref
-from collections import OrderedDict
+from collections import OrderedDict, defaultdict
 from itertools import chain
 from pathlib import Path
 from typing import (
@@ -51,7 +51,7 @@ from robotcode.language_server.robotframework.utils.ast_utils import (
     tokenize_variables,
 )
 from robotcode.language_server.robotframework.utils.async_ast import Visitor
-from robotcode.language_server.robotframework.utils.match import eq
+from robotcode.language_server.robotframework.utils.match import eq_namespace
 from robotcode.language_server.robotframework.utils.variables import BUILTIN_VARIABLES
 from robotcode.language_server.robotframework.utils.version import get_robot_version
 
@@ -191,7 +191,8 @@ class BlockVariableVisitor(Visitor):
         from robot.parsing.lexer.tokens import Token as RobotToken
         from robot.parsing.model.statements import KeywordName
         from robot.variables.search import search_variable
-        from robotcode.language_server.robotframework.parts.model_helper import ModelHelperMixin
+
+        from .model_helper import ModelHelperMixin
 
         n = cast(KeywordName, node)
         name_token = cast(Token, n.get_token(RobotToken.KEYWORD_NAME))
@@ -538,6 +539,7 @@ class Namespace:
         self.workspace_languages = workspace_languages
 
         self._libraries: OrderedDict[str, LibraryEntry] = OrderedDict()
+        self._namespaces: Optional[Dict[KeywordMatcher, List[LibraryEntry]]] = None
         self._libraries_matchers: Optional[Dict[KeywordMatcher, LibraryEntry]] = None
         self._resources: OrderedDict[str, ResourceEntry] = OrderedDict()
         self._resources_matchers: Optional[Dict[KeywordMatcher, ResourceEntry]] = None
@@ -661,6 +663,7 @@ class Namespace:
     async def _invalidate(self) -> None:
         self._initialized = False
 
+        self._namespaces = None
         self._libraries = OrderedDict()
         self._libraries_matchers = None
         self._resources = OrderedDict()
@@ -731,33 +734,26 @@ class Namespace:
 
         return self._import_entries
 
-    @_logger.call
     async def get_libraries(self) -> OrderedDict[str, LibraryEntry]:
         await self.ensure_initialized()
 
         return self._libraries
 
-    async def get_libraries_matchers(self) -> Dict[KeywordMatcher, LibraryEntry]:
-        if self._libraries_matchers is None:
-            self._libraries_matchers = {
-                KeywordMatcher(v.alias or v.name or v.import_name, False): v
-                for v in (await self.get_libraries()).values()
-            }
-        return self._libraries_matchers
+    async def get_namespaces(self) -> Dict[KeywordMatcher, List[LibraryEntry]]:
+        if self._namespaces is None:
+            self._namespaces = defaultdict(list)
+
+            for v in (await self.get_libraries()).values():
+                self._namespaces[KeywordMatcher(v.alias or v.name or v.import_name, is_namespace=True)].append(v)
+            for v in (await self.get_resources()).values():
+                self._namespaces[KeywordMatcher(v.alias or v.name or v.import_name, is_namespace=True)].append(v)
+        return self._namespaces
 
     async def get_resources(self) -> OrderedDict[str, ResourceEntry]:
         await self.ensure_initialized()
 
         return self._resources
 
-    async def get_resources_matchers(self) -> Dict[KeywordMatcher, ResourceEntry]:
-        if self._resources_matchers is None:
-            self._resources_matchers = {
-                KeywordMatcher(v.alias or v.name or v.import_name, False): v
-                for v in (await self.get_resources()).values()
-            }
-        return self._resources_matchers
-
     async def get_imported_variables(self) -> OrderedDict[str, VariablesEntry]:
         await self.ensure_initialized()
 
@@ -1578,8 +1574,6 @@ class Namespace:
                         self,
                         await self.create_finder(),
                         self.get_ignored_lines(self.document) if self.document is not None else [],
-                        await self.get_libraries_matchers(),
-                        await self.get_resources_matchers(),
                     ).run()
 
                     self._diagnostics += result.diagnostics
@@ -1839,13 +1833,13 @@ class KeywordFinder:
         if get_robot_version() >= (6, 0):
             result: List[Tuple[LibraryEntry, KeywordDoc]] = []
             for v in self._all_keywords:
-                if eq(v.alias or v.name, owner_name):
+                if eq_namespace(v.alias or v.name, owner_name):
                     result.extend((v, kw) for kw in v.library_doc.keywords.get_all(name))
             return result
 
         result = []
         for v in self._all_keywords:
-            if eq(v.alias or v.name, owner_name):
+            if eq_namespace(v.alias or v.name, owner_name):
                 kw = v.library_doc.keywords.get(name, None)
                 if kw is not None:
                     result.append((v, kw))
@@ -1973,7 +1967,7 @@ class KeywordFinder:
     ) -> List[Tuple[Optional[LibraryEntry], KeywordDoc]]:
         for libname in self.namespace.search_order:
             for e in entries:
-                if e[0] is not None and eq(libname, e[0].alias or e[0].name):
+                if e[0] is not None and eq_namespace(libname, e[0].alias or e[0].name):
                     return [e]
 
         return entries
