@@ -32,10 +32,10 @@ from robotcode.core.lsp.types import (
     SemanticTokensPartialResult,
     SemanticTokenTypes,
 )
-from robotcode.language_server.common.decorators import language_id
-from robotcode.language_server.common.text_document import TextDocument, range_to_utf16
-from robotcode.language_server.robotframework.diagnostics.entities import LibraryEntry, ResourceEntry
-from robotcode.language_server.robotframework.diagnostics.library_doc import (
+
+from ...common.decorators import language_id
+from ...common.text_document import TextDocument, range_to_utf16
+from ..diagnostics.library_doc import (
     ALL_RUN_KEYWORDS_MATCHERS,
     BUILTIN_LIBRARY_NAME,
     KeywordArgumentKind,
@@ -43,25 +43,16 @@ from robotcode.language_server.robotframework.diagnostics.library_doc import (
     KeywordMatcher,
     LibraryDoc,
 )
-from robotcode.language_server.robotframework.diagnostics.namespace import (
-    DEFAULT_BDD_PREFIXES,
-    Namespace,
-)
-from robotcode.language_server.robotframework.utils import async_ast
-from robotcode.language_server.robotframework.utils.ast_utils import (
-    HasTokens,
-    Token,
-    iter_over_keyword_names_and_owners,
-    token_in_range,
-)
-from robotcode.language_server.robotframework.utils.version import get_robot_version
-
-from .model_helper import ModelHelperMixin
+from ..diagnostics.model_helper import ModelHelperMixin
+from ..diagnostics.namespace import DEFAULT_BDD_PREFIXES, Namespace
+from ..utils import async_ast
+from ..utils.ast_utils import HasTokens, Token, iter_over_keyword_names_and_owners, token_in_range
+from ..utils.version import get_robot_version
+from .protocol_part import RobotLanguageServerProtocolPart
 
 if TYPE_CHECKING:
     from robotcode.language_server.robotframework.protocol import RobotLanguageServerProtocol
 
-from .protocol_part import RobotLanguageServerProtocolPart
 
 ROBOT_KEYWORD_INNER = "KEYWORD_INNER"
 ROBOT_NAMED_ARGUMENT = "NAMED_ARGUMENT"
@@ -261,15 +252,13 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
     )
     BDD_TOKEN_REGEX = re.compile(r"^(Given|When|Then|And|But)\s", flags=re.IGNORECASE)
 
-    BUILTIN_MATCHER = KeywordMatcher("BuiltIn")
+    BUILTIN_MATCHER = KeywordMatcher("BuiltIn", is_namespace=True)
 
     @classmethod
     async def generate_sem_sub_tokens(
         cls,
         namespace: Namespace,
         builtin_library_doc: Optional[LibraryDoc],
-        libraries_matchers: Dict[KeywordMatcher, LibraryEntry],
-        resources_matchers: Dict[KeywordMatcher, ResourceEntry],
         token: Token,
         node: ast.AST,
         col_offset: Optional[int] = None,
@@ -382,20 +371,18 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
 
                 kw_namespace: Optional[str] = None
                 kw: str = token.value
+                kw_doc = await namespace.find_keyword(token.value, raise_keyword_error=False)
 
-                for lib, name in iter_over_keyword_names_and_owners(token.value):
-                    if lib is not None and (
-                        any(k for k in libraries_matchers.keys() if k == lib)
-                        or any(k for k in resources_matchers.keys() if k == lib)
-                    ):
-                        kw_namespace = lib
-                        if name:
-                            kw = name
-                        break
+                lib_entry, kw_namespace = await cls.get_namespace_info_from_keyword_token(namespace, token)
+                if lib_entry is not None and kw_doc:
+                    if kw_doc.parent != lib_entry.library_doc:
+                        kw_namespace = None
 
-                kw_index = token.value.index(kw)
+                kw_index = len(kw_namespace) + 1 if kw_namespace else 0
 
                 if kw_namespace:
+                    kw = token.value[kw_index + 1 :]
+
                     yield SemTokenInfo(
                         token.lineno,
                         col_offset,
@@ -411,13 +398,11 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
                     )
 
                 if builtin_library_doc is not None and kw in builtin_library_doc.keywords:
-                    doc = await namespace.find_keyword(token.value)
-                    if doc is not None and doc.libname == cls.BUILTIN_MATCHER and doc.matcher == kw:
+                    if kw_doc is not None and kw_doc.libname == cls.BUILTIN_MATCHER and kw_doc.matcher == kw:
                         if not sem_mod:
                             sem_mod = set()
                         sem_mod.add(RobotSemTokenModifiers.BUILTIN)
 
-                kw_doc = await namespace.find_keyword(token.value, raise_keyword_error=False)
                 if kw_doc is not None and kw_doc.is_embedded:
                     if get_robot_version() >= (6, 0):
                         m = kw_doc.matcher.embedded_arguments.match(kw)
@@ -492,8 +477,6 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
         node: ast.AST,
         namespace: Namespace,
         builtin_library_doc: Optional[LibraryDoc],
-        libraries_matchers: Dict[KeywordMatcher, LibraryEntry],
-        resources_matchers: Dict[KeywordMatcher, ResourceEntry],
     ) -> AsyncIterator[SemTokenInfo]:
         from robot.parsing.lexer.tokens import Token as RobotToken
         from robot.parsing.model.statements import Arguments, Variable
@@ -542,23 +525,17 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
                 ignore_errors=True,
                 identifiers="$" if token.type == RobotToken.KEYWORD_NAME else "$@&%",
             ):
-                async for e in self.generate_sem_sub_tokens(
-                    namespace, builtin_library_doc, libraries_matchers, resources_matchers, sub_token, node
-                ):
+                async for e in self.generate_sem_sub_tokens(namespace, builtin_library_doc, sub_token, node):
                     yield e
 
         else:
-            async for e in self.generate_sem_sub_tokens(
-                namespace, builtin_library_doc, libraries_matchers, resources_matchers, token, node
-            ):
+            async for e in self.generate_sem_sub_tokens(namespace, builtin_library_doc, token, node):
                 yield e
 
     async def generate_run_kw_tokens(
         self,
         namespace: Namespace,
         builtin_library_doc: Optional[LibraryDoc],
-        libraries_matchers: Dict[KeywordMatcher, LibraryEntry],
-        resources_matchers: Dict[KeywordMatcher, ResourceEntry],
         kw_doc: Optional[KeywordDoc],
         kw_token: Token,
         arguments: List[Token],
@@ -584,8 +561,6 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
                 async for b in self.generate_run_kw_tokens(
                     namespace,
                     builtin_library_doc,
-                    libraries_matchers,
-                    resources_matchers,
                     await namespace.find_keyword(unescape(token.value), raise_keyword_error=False),
                     RobotToken(ROBOT_KEYWORD_INNER, token.value, token.lineno, token.col_offset, token.error),
                     arguments[1:],
@@ -606,8 +581,6 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
                     async for b in self.generate_run_kw_tokens(
                         namespace,
                         builtin_library_doc,
-                        libraries_matchers,
-                        resources_matchers,
                         await namespace.find_keyword(unescape(token.value), raise_keyword_error=False),
                         RobotToken(ROBOT_KEYWORD_INNER, token.value, token.lineno, token.col_offset, token.error),
                         arguments[1:],
@@ -646,8 +619,6 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
                     async for e in self.generate_run_kw_tokens(
                         namespace,
                         builtin_library_doc,
-                        libraries_matchers,
-                        resources_matchers,
                         await namespace.find_keyword(unescape(token.value), raise_keyword_error=False),
                         RobotToken(ROBOT_KEYWORD_INNER, token.value, token.lineno, token.col_offset, token.error),
                         args,
@@ -718,8 +689,6 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
                         async for e in self.generate_run_kw_tokens(
                             namespace,
                             builtin_library_doc,
-                            libraries_matchers,
-                            resources_matchers,
                             inner_kw_doc,
                             RobotToken(ROBOT_KEYWORD_INNER, token.value, token.lineno, token.col_offset, token.error),
                             args,
@@ -754,7 +723,7 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
                         kw_doc = await namespace.find_keyword(kw_token.value)
 
                     if kw_doc and any(
-                        v for v in kw_doc.args if v.kind == KeywordArgumentKind.VAR_NAMED or v.name == name
+                        v for v in kw_doc.arguments if v.kind == KeywordArgumentKind.VAR_NAMED or v.name == name
                     ):
                         length = len(name)
                         yield RobotToken(ROBOT_NAMED_ARGUMENT, name, token.lineno, token.col_offset), node
@@ -776,8 +745,6 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
         range: Optional[Range],
         namespace: Namespace,
         builtin_library_doc: Optional[LibraryDoc],
-        libraries_matchers: Dict[KeywordMatcher, LibraryEntry],
-        resources_matchers: Dict[KeywordMatcher, ResourceEntry],
     ) -> Union[SemanticTokens, SemanticTokensPartialResult, None]:
         from robot.parsing.lexer.tokens import Token as RobotToken
         from robot.parsing.model.statements import Fixture, KeywordCall, LibraryImport, VariablesImport
@@ -800,10 +767,10 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
                                     if (
                                         value is not None
                                         and kw_doc is not None
-                                        and kw_doc.args
+                                        and kw_doc.arguments
                                         and any(
                                             v
-                                            for v in kw_doc.args
+                                            for v in kw_doc.arguments
                                             if v.kind == KeywordArgumentKind.VAR_NAMED or v.name == name
                                         )
                                     ):
@@ -837,10 +804,10 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
                                     if (
                                         value is not None
                                         and kw_doc is not None
-                                        and kw_doc.args
+                                        and kw_doc.arguments
                                         and any(
                                             v
-                                            for v in kw_doc.args
+                                            for v in kw_doc.arguments
                                             if v.kind == KeywordArgumentKind.VAR_NAMED or v.name == name
                                         )
                                     ):
@@ -893,8 +860,6 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
                                     async for kw_res in self.generate_run_kw_tokens(
                                         namespace,
                                         builtin_library_doc,
-                                        libraries_matchers,
-                                        resources_matchers,
                                         kw_doc,
                                         kw_token,
                                         node.tokens[node.tokens.index(kw_token) + 1 :],
@@ -929,9 +894,7 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
                 ),
             ),
         ):
-            async for token in self.generate_sem_tokens(
-                robot_token, robot_node, namespace, builtin_library_doc, libraries_matchers, resources_matchers
-            ):
+            async for token in self.generate_sem_tokens(robot_token, robot_node, namespace, builtin_library_doc):
                 if token.length == 0:
                     continue
 
@@ -992,15 +955,7 @@ class RobotSemanticTokenProtocolPart(RobotLanguageServerProtocolPart, ModelHelpe
             None,
         )
 
-        return await self._collect_internal(
-            document,
-            model,
-            range,
-            namespace,
-            builtin_library_doc,
-            await namespace.get_libraries_matchers(),
-            await namespace.get_resources_matchers(),
-        )
+        return await self._collect_internal(document, model, range, namespace, builtin_library_doc)
 
     @language_id("robotframework")
     @_logger.call
